from .file_tree import FileTree
from .file_tree_builder import build_from_directory
from .bmv_parser import grammar, FileTreeVisitor
from .helpers import get_last_index_of_substring
from .operations import perform_adds, perform_deletes, perform_renames, perform_moves

__version__ = "0.1.1"



import unittest
from bulk_mv import build_from_directory, FileTree
from pathlib import Path


class TestFileTreeBuilder(unittest.TestCase):
    def build_sample_1(self):
        sample1 = build_from_directory(f"{Path(__file__).parent}/dummy_directories/sample1/")
        return sample1

    def build_sample_2(self):
        sample2 = build_from_directory(f"{Path(__file__).parent}/dummy_directories/sample2/")
        return sample2

    def test_build_from_directory_trivial(self):
        sample2 = self.build_sample_2()

        ft = FileTree("sample2")
        ft.add_child(FileTree("a.txt"))
        ft.add_child(FileTree("b.txt"))
        ft.add_child(FileTree("c.txt"))
        ft.add_child(FileTree("readme.md"))

        data_folder = FileTree("data")
        data_folder.add_child(FileTree("report.csv"))

        web_folder = FileTree("web")
        web_folder.add_child(FileTree("index.html"))
        web_folder.add_child(FileTree("main.css"))
        web_folder.add_child(FileTree("main.js"))

        ft.add_child(data_folder)
        ft.add_child(web_folder)

        self.assertEqual(sample2, ft)



from parsimonious.grammar import Grammar
from parsimonious.nodes import NodeVisitor

grammar = Grammar(
    """
    line = ((unary_operation / binary_operation / entity) ws nl?)*
    entity = (directory / filename)
    unary_operation = (add / delete)
    binary_operation = (rename / move)

    directory  = ~"[a-zA-Z0-9/_\-\.]+"
    filename = ~"[A-Za-z0-9\./_\-]+"

    rename_op = "->"
    rename = entity rws rename_op rws entity

    move_op = "=>"
    move = entity rws move_op rws directory

    add_op = "+"
    add = add_op rws entity

    delete_op = "-"
    delete = delete_op rws entity

    rws = ~"\s+"
    ws = ~"\s*"
    nl = ~"\\n"
"""
)


class FileTreeVisitor(NodeVisitor):
    def visit_line(self, node, visited_children):
        output = {"move": [], "rename": [], "add": [], "delete": []}  # , "dir": [], "filename": []}
        for child in visited_children:
            if child[0][0] is not None:
                for key, val in child[0][0].items():
                    output[key].append(val)

        return output

    def visit_unary_operation(self, node, visited_children):
        return {node.children[0].expr_name: visited_children[0]}

    def visit_add(self, node, visited_children):
        _, *_, new_entity = node.children
        new_name = new_entity.text.strip()
        return new_name

    def visit_delete(self, node, visited_children):
        _, *_, old_entity = node.children
        old_name = old_entity.text.strip()
        return old_name

    def visit_binary_operation(self, node, visited_children):
        return {node.children[0].expr_name: visited_children[0]}

    def visit_move(self, node, visited_children):
        old_entity_name, _, move_op, *_, new_entity_name = node.children

        old_name = old_entity_name.children[0].text.strip()
        new_name = new_entity_name.text.strip()
        return {"current_path": old_name, "new_path": new_name}

    def visit_rename(self, node, visited_children):
        old_entity_name, _, rename_op, *_, new_entity_name = node.children
        old_name = old_entity_name.text.strip()
        new_name = new_entity_name.text.strip()
        return {"old_name": old_name, "new_name": new_name}

    def visit_entity(self, node, visited_children):
        # return visited_children[0][0]
        return None

    def visit_directory(self, node, visited_children):
        return {"dir": node.text}

    def visit_filename(self, node, visited_children):
        return {"filename": node.text}

    def visit_ws(self, node, visited_children):
        return ""

    def visit_move_op(self, node, visited_children):
        return ""

    def visit_rename_op(self, node, visited_children):
        return ""

    def visit_delete_op(self, node, visited_children):
        return ""

    def visit_add_op(self, node, visited_children):
        return ""

    def generic_visit(self, node, visited_children):
        """The generic visit method."""
        return visited_children or node



from collections import deque
from .helpers import get_last_index_of_substring


class FileTree:
    def __init__(self, path):
        self.path = path if path[-1] != "/" else path[:-1]
        self.children = []
        self.parent = None

    def all_paths(self):
        stack = [self]
        paths = []

        while stack:
            tree = stack.pop()
            paths.append(tree._path() + "/")

            for child in tree.children:
                if child._is_dir():
                    stack.append(child)
                else:
                    paths.append(child._path())

        return paths

    def add_child(self, child):
        self.children.append(child)
        child.parent = self
        child._set_path(f"{self._path()}/{child._name()}")

        def _update_paths_for_children(tree):
            if tree._children():
                stack = [tree]
                while stack:
                    tree = stack.pop()

                    for tree in tree.children:
                        tree._set_path(f"{tree._parent()._path()}/{tree._name()}")
                        stack.append(tree)

        _update_paths_for_children(child)

    def _set_path(self, path):
        self.path = path

    def _children(self):
        return self.children

    def _parent(self):
        return self.parent

    def _path(self):
        return self.path

    def _is_dir(self):
        return len(self.children) > 0

    def _name(self):
        try:
            last_slash_index = get_last_index_of_substring(self._path(), "/")
            return self._path()[last_slash_index + 1 :]
        except ValueError:
            return self._path()

    def __eq__(self, other):
        stack = deque([self])
        other_stack = deque([other])

        while stack and other_stack:
            tree = stack.pop()
            other_tree = other_stack.pop()

            if len(tree.children) != len(other_tree.children):
                return False

            for child in tree.children:
                other_child = None
                for other_child in other_tree.children:
                    if child._name() == other_child._name():
                        break

                if not other_child or child._name() != other_child._name():
                    return False

                if len(child.children) != len(other_child.children):
                    return False

                if child._is_dir():
                    stack.append(child)
                    other_stack.append(other_child)

        return len(stack) == 0 and len(stack) == len(other_stack)



from .file_tree_builder import build_from_directory
from .bmv_parser import grammar, FileTreeVisitor
from os import system
from .operations import perform_adds, perform_deletes, perform_renames, perform_moves


def run(start_path):
    ft = build_from_directory(start_path)
    paths = ft.all_paths()

    with open("file_tree.bmv", "w") as file:
        file.write('\n'.join(paths))

    system("vim file_tree.bmv")

    with open("file_tree.bmv", "r") as file:
        tree = grammar.parse(file.read().strip())
        ftv = FileTreeVisitor()
        output = ftv.visit(tree)

        # print(json.dumps(output, indent=4))

    perform_adds(output["add"])
    perform_deletes(output["delete"])
    perform_renames(output["rename"])
    perform_moves(output["move"])

    perform_deletes(['file_tree.bmv'])

    return 0



#!/usr/bin/env python

from setuptools import find_packages
from setuptools import setup

setup(
    name='bulk-mv',
    version="0.1.1",
    description='Move, rename, delete, and add files as fast as you can use Vim.',
    author='Angel Garcia',
    author_email='angarc37@gmail.com',
    url='https://github.com/angarc/bulk-mv',
    py_modules=[
        'bulk_mv.__main__',
    ],
    packages=find_packages(),
    # install_requires=[
    # ],
    entry_points={'console_scripts': ['bmv = bulk_mv.__main__:main']},
)



from os import listdir
from os.path import isdir
from .file_tree import FileTree

ignore = ['.git', '.github', '.pytest-cache', '__pycache__', '.pytest_cache', 'node_modules', 'tmp']


def build_from_directory(path):
    file_tree = FileTree(path)
    ignore_set = set(ignore)

    for entity_name in listdir(path):
        new_path = f"{path}/{entity_name}"

        child_file_tree = None
        if isdir(new_path) and entity_name not in ignore_set:
            child_file_tree = build_from_directory(new_path)
        else:
            child_file_tree = FileTree(new_path)

        if entity_name not in ignore_set:
            file_tree.add_child(child_file_tree)

    return file_tree



import unittest
from bulk_mv import build_from_directory, FileTree, grammar, FileTreeVisitor
from pathlib import Path
import json


class TestBmvParser(unittest.TestCase):
    def sample1_file_contents(self):
        with open(f"{Path(__file__).parent}/dummy_bmv_files/sample1.bmv") as file:
            return file.read().strip()

        return None

    def test_file_tree_visitor(self):
        sample1 = self.sample1_file_contents()
        tree = grammar.parse(sample1)

        ftv = FileTreeVisitor()
        output = ftv.visit(tree)

        expected_output = {
            "move": [
                {"current_path": "web/static/", "new_path": "web/assets/"},
                {"current_path": "web/images/", "new_path": "web/static/assets/images/"},
                {"current_path": "web/images/photo.jpg", "new_path": "web/pages/images/profile_photos"},
            ],
            "rename": [
                {"old_name": "web/static/main.css", "new_name": "script.css"},
                {"old_name": "web/static/main.js", "new_name": "script.js"},
            ],
            "add": ["web/pages/images/profile_photos", "web/assets/"],
            "delete": ["web/images/delete_me.jpg"],
        }

        self.assertEqual(output, expected_output)



import unittest
from bulk_mv import get_last_index_of_substring


class TestHelpers(unittest.TestCase):
    def test_get_last_index_of_substring_normal(self):
        file_path = "/path/to/some/file"

        last_index = get_last_index_of_substring(file_path, "/")

        self.assertEqual(last_index, 13)

    def test_get_last_index_of_substring_edge(self):
        string = "////////"

        last_index = get_last_index_of_substring(string, "/")

        self.assertEqual(last_index, 7)

    def test_get_last_index_of_substring_trivial(self):
        string = "/"

        last_index = get_last_index_of_substring(string, "/")

        self.assertEqual(last_index, 0)
